<?php
namespace FORM4\Form4Pages\Helper;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @author	Johannes Härtner <johannes.haertner@form4.de>
 * @package form4_pages
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class DoktypeManager implements \TYPO3\CMS\Extbase\Persistence\RepositoryInterface {

	/**
	 * @var \TYPO3\CMS\Extbase\Object\ObjectManagerInterface
	 */
	protected $objectManager;

	/**
	 * doktypeManager
	 *
	 * @var \FORM4\Form4Pages\Utility\ConfigurationUtility
	 * @inject
	 */
	protected $configurationUtility;

	/**
	 * @var \TYPO3\CMS\Extbase\Persistence\Generic\Session
	 * @inject
	 */
	protected $session;

	/**
	 * @var \TYPO3\CMS\Extbase\Persistence\PersistenceManagerInterface
	 * @inject
	 */
	protected $persistenceManager;

	/**
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
	 * @inject
	 */
	protected $configurationManager;

	/**
	 * @var \TYPO3\CMS\Extbase\Persistence\Generic\Mapper\DataMapper
	 * @inject
	 */
	protected $dataMapper;

	/**
	 * contains all entities mapped to table pages except abstract classes
	 *
	 * @var array
	 */
	protected $doktypes = array();

	/**
	 * This array assigns an entity to every doktype. It defines which class is returned for a specific doktype.
	 *
	 * @var array
	 */
	protected $doktypeEntities = array();

	/**
	 *
	 * @var \string
	 */
	protected $generalPageEntity = '';

	/**
	 * @var array
	 */
	protected $defaultOrderings = array();

	/**
	 * @var \TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface
	*/
	protected $defaultQuerySettings = NULL;

	//TODO __call: its probably better to exclude doktypes without [Property]. Use ReflectionClass::hasProperty to check classes. query->in for doktype filter
	//TODO implement some kind of strict mode where only doktypes are used which are in $doktypeEntities. This could mean: no 'default' class in $doktypeEntities. Requires more methods for $doktypeEntities: remove(doktype), setWholeArray().
	//TODO maybe we can find better class name and property names

	/**
	 * Create new instance
	 *
	 * @param \TYPO3\CMS\Extbase\Object\ObjectManagerInterface $objectManager
	 */
	public function __construct(\TYPO3\CMS\Extbase\Object\ObjectManagerInterface $objectManager = NULL) {
		if ($objectManager === NULL) {
			$this->objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Object\\ObjectManager');
		} else {
			$this->objectManager = $objectManager;
		}

		$this->initializeDoktypes();
		$this->initializeDefaultDoktypeEntities();
		$this->generalPageEntity = 'FORM4\Form4Pages\Domain\Model\Page';

		$defaultQuerySettings = $this->objectManager->create('TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface');
		$defaultQuerySettings->setRespectStoragePage(FALSE);
		$this->setDefaultQuerySettings($defaultQuerySettings);

	}

	/**
	 * Gets all classes which are mapped to table 'pages' (except abstract classes) using the TSFE.
	 *
	 * @return void
	 */
	protected function initializeDoktypes() {
		//config.tx_extbase.persistence.classes
		if (TYPO3_MODE == 'BE') {
			$backendConfigurarionManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Configuration\\BackendConfigurationManager');
			$fullConfiguration = $backendConfigurarionManager->getTypoScriptSetup();
			$extbaseConfig = $fullConfiguration['config.']['tx_extbase.']['persistence.']['classes.'];
		} else {
// 			$fullConfiguration = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);
			$extbaseConfig = $GLOBALS['TSFE']->tmpl->setup['config.']['tx_extbase.']['persistence.']['classes.'];
		}

		foreach ($extbaseConfig as $className => $classConfiguration) {
			$className = substr($className, 0, -1); //remove dot at the end
			if ($classConfiguration['mapping.']['tableName'] === 'pages' && !$this->isAbstractClass($className)) {
				$this->doktypes[($classConfiguration['mapping.']['recordType']) ? ($classConfiguration['mapping.']['recordType']) : 'default'][] = $className ;
			}
		}
	}

	/**
	 * Sets a default entity class for to each doktype. It takes the latest configured class.
	 * The array can be modified by setDoktypeEntity().
	 *
	 * @see setDoktypeEntity()
	 * @return void
	 */
	protected function initializeDefaultDoktypeEntities() {
		foreach ($this->doktypes as $doktype => $className) {
			$this->doktypeEntities[$doktype] = end($this->doktypes[$doktype]);
		}
	}


	/**
	 * Checks if the given class is abstract.
	 *
	 * @param string $className Name of the class.
	 * @return boolean
	 */
	protected function isAbstractClass($className) {
		$reflector = new \ReflectionClass($className);
		return $reflector->isAbstract();
	}

	/**
	 * Checks if the given object is an instance of at least one of the doktypes determined on construction.
	 *
	 * @param mixed $entity
	 * @return boolean
	 */
	protected function isPageEntity($entity) {
		$entity = is_object($entity) ? get_class($entity) : $entity;
		foreach ($this->doktypes as $doktype => $classes) {
			if (in_array($entity, $classes)) {
				return TRUE;
			}
		}
		return FALSE;
	}

	/**
	 * Returns the $doktypes array. This array contains all entities mapped to table pages except abstract classes
	 *
	 * @return array
	 */
	public function getDoktypes() {
		return $this->doktypes;
	}

	/**
	 * Returns the doktype configuration array. This array defines which class to return for a specific doktype.
	 *
	 * @return array
	 */
	public function getDoktypeEntities() {
		return $this->doktypeEntities;
	}

	/**
	 * Sets a specific entity class for each doktype.
	 *
	 * @param integer $doktype the doktype number
	 * @param string $entityClassName class name
	 */
	public function setDoktypeEntity($doktype, $entityClassName) {
		if (!$this->isPageEntity($entityClassName)) {
			throw new \TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException('The entityClassName given to setDoktypeEntity() was not a page type.', 1390220177);
		}
		$this->doktypeEntities[$doktype] = $entityClassName;
	}

	public function getDoktypeEntity($doktype) {
		return !empty($this->doktypeEntities[$doktype]) ? $this->doktypeEntities[$doktype] : $this->doktypeEntities['default'];
	}

	/**
	 * Adds a page object.
	 *
	 * @param object $object The object to add
	 * @return void
	 * @api
	 */
	public function add($object) {
		if (!$this->isPageEntity($object)) {
			throw new \TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException('The object given to remove() was not a page type.', 1390223538);
		}
		$this->persistenceManager->add($object);
	}

	/**
	 * Removes a page object.
	 *
	 * @param object $object The object to remove
	 * @return void
	 * @api
	*/
	public function remove($object) {
		if (!$this->isPageEntity($object)) {
			throw new \TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException('The object given to remove() was not a page type.', 1390223565);
		}
		$this->persistenceManager->remove($object);
	}

	/**
	 * Replaces an existing object with the same identifier by the given object.
	 *
	 * @param object $modifiedObject The modified object
	 * @api
	*/
	public function update($modifiedObject) {
		if (!$this->isPageEntity($modifiedObject)) {
			throw new \TYPO3\CMS\Extbase\Persistence\Exception\IllegalObjectTypeException('The object given to remove() was not a page type.', 1390223572);
		}
		$this->persistenceManager->update($modifiedObject);
	}

	/**
	 * Returns all pages.
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface|array The query result
	 * @api
	*/
	public function findAll() {
		return $this->createQuery()->execute();
	}

	/**
	 * Returns the total number of pages.
	 *
	 * @return integer The object count
	 * @api
	*/
	public function countAll() {
		return $this->createQuery()->execute()->count();
	}

	/**
	 * Removes all objects as if remove() was called for each of them.
	 *
	 * @return void
	 * @api
	*/
	public function removeAll() {
		foreach ($this->findAll() AS $object) {
			$this->remove($object);
		}
	}

	/**
	 * Finds an object matching the given identifier.
	 *
	 * @param integer $uid The identifier of the object to find
	 * @return object The matching object if found, otherwise NULL
	 * @api
	*/
	public function findByUid($uid) {
		return $this->findByIdentifier($uid);
	}

	/**
	 * Finds an object matching the given identifier.
	 *
	 * @param mixed $identifier The identifier of the object to find
	 * @return object The matching object if found, otherwise NULL
	 * @api
	*/
	public function findByIdentifier($identifier) {
		return $this->findByIdentifierAndDoktype($identifier, $this->generalPageEntity);
	}

	/**
	 * Finds an object by its identifier and doktype.
	 *
	 * @param mixed $identifier
	 * @param integer $doktype
	 * @return object
	 */
	public function findByIdentifierAndDoktype($identifier, $doktype) {
		$query = $this->createQuery();
		$query->getQuerySettings()->setRespectStoragePage(FALSE);
		$query->getQuerySettings()->setRespectSysLanguage(FALSE);
		$object = $query->matching($query->equals('uid', $identifier))->execute()->getFirst();
		return $object;
	}

	/**
	 * Sets the property names the results may be sorted by per default.
	 * Expected like this:
	 * array(
	 *   'foo' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING,
	 *   'bar' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING
	 * )
	 *
	 * @param array $defaultOrderings The property names to order by
	 * @return void
	 * @api
	*/
	public function setDefaultOrderings(array $defaultOrderings) {
		$this->defaultOrderings = $defaultOrderings;
	}

	/**
	 * Sets the default query settings to be used in this repository.
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface $defaultQuerySettings
	 * @return void
	 * @api
	*/
	public function setDefaultQuerySettings(\TYPO3\CMS\Extbase\Persistence\Generic\QuerySettingsInterface $defaultQuerySettings) {
		$this->defaultQuerySettings = $defaultQuerySettings;
	}

	/**
	 * Returns a query for objects of this repository.
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\QueryInterface
	 * @api
	*/
	public function createQuery() {
		$query = $this->objectManager->get('FORM4\\Form4Pages\\Persistence\\Query', $this->generalPageEntity, $this->doktypeEntities, TRUE);
		$querySettings = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\QuerySettingsInterface');

		$dataMap = $this->dataMapper->getDataMap($this->generalPageEntity);

		if ($dataMap->getIsStatic() || $dataMap->getRootLevel()) {
			$querySettings->setRespectStoragePage(FALSE);
		}

		$frameworkConfiguration = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);
		$querySettings->setStoragePageIds(\TYPO3\CMS\Core\Utility\GeneralUtility::intExplode(',', $frameworkConfiguration['persistence']['storagePid']));
		$query->setQuerySettings($querySettings);



		if ($this->defaultOrderings !== array()) {
			$query->setOrderings($this->defaultOrderings);
		}
		if ($this->defaultQuerySettings !== NULL) {
			$query->setQuerySettings(clone $this->defaultQuerySettings);
		}
		return $query;
	}

	/**
	 * Dispatches magic methods (findBy[Property]()).
	 *
	 * @param string $methodName The name of the magic method
	 * @param string $arguments The arguments of the magic method
	 * @throws \TYPO3\CMS\Extbase\Persistence\Generic\Exception\UnsupportedMethodException
	 * @return mixed
	 * @api
	 */
	public function __call($methodName, $arguments) {
		if (substr($methodName, 0, 6) === 'findBy' && strlen($methodName) > 7) {
			$propertyName = lcfirst(substr($methodName, 6));
			$query = $this->createQuery();
			$result = $query->matching($query->equals($propertyName, $arguments[0]))->execute();
			return $result;
		} elseif (substr($methodName, 0, 9) === 'findOneBy' && strlen($methodName) > 10) {
			$propertyName = lcfirst(substr($methodName, 9));
			$query = $this->createQuery();

			$result = $query->matching($query->equals($propertyName, $arguments[0]))->setLimit(1)->execute();
			if ($result instanceof \TYPO3\CMS\Extbase\Persistence\QueryResultInterface) {
				return $result->getFirst();
			} elseif (is_array($result)) {
				return isset($result[0]) ? $result[0] : NULL;
			}

		} elseif (substr($methodName, 0, 7) === 'countBy' && strlen($methodName) > 8) {
			$propertyName = lcfirst(substr($methodName, 7));
			$query = $this->createQuery();
			$result = $query->matching($query->equals($propertyName, $arguments[0]))->execute()->count();
			return $result;
		}
		throw new \TYPO3\CMS\Extbase\Persistence\Generic\Exception\UnsupportedMethodException('The method "' . $methodName . '" is not supported.', 1390223589);
	}
}
?>