<?php
namespace FORM4\Form4Pages\Persistence;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @author	Johannes Härtner <johannes.haertner@form4.de>
 * @package form4_pages
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Query extends \TYPO3\CMS\Extbase\Persistence\Generic\Query {

	/**
	 * This array assigns an entity to every doktype. It defines which class is returned for a specific doktype.
	 *
	 * @var array
	 */
	protected $doktypeEntities = array();

	/**
	 *
	 * @var \string
	 */
	protected $type = '';

	/**
	 *
	 * @var \boolean
	 */
	protected $resolveEntitiesToDoktype = FALSE;

	/**
	 * @var \TYPO3\CMS\Frontend\Page\PageRepository
	 * @inject
	 */
	protected $pageRepository;

	/**
	 * Constructs a query object working on the given class name
	 *
	 * @param string $type
	 * @param array $doktypeEntities
	 */
	public function __construct($type, $doktypeEntities = array(), $resolveEntitiesToDoktype = FALSE) {
		parent::__construct($type);
		$this->doktypeEntities = $doktypeEntities;
		$this->resolveEntitiesToDoktype = $resolveEntitiesToDoktype;
	}

	protected function getDoktypeEntity($doktype) {
		return !empty($this->doktypeEntities[$doktype]) ? $this->doktypeEntities[$doktype] : $this->doktypeEntities['default'];
	}

	/**
	 * Executes the query against the database and returns the result
	 *
	 * @param $returnRawQueryResult boolean avoids the object mapping by the persistence
	 * @return \TYPO3\CMS\Extbase\Persistence\QueryResultInterface|array The query result object or an array if $returnRawQueryResult is TRUE
	 * @api
	 */
	public function execute($returnRawQueryResult = FALSE, $queryResultClass = 'FORM4\\Form4Pages\\Persistence\\QueryResult', $constructorParams = array()) {
		if ($returnRawQueryResult === TRUE || $this->getQuerySettings()->getReturnRawQueryResult() === TRUE) {
			return $this->persistenceManager->getObjectDataByQuery($this);
		} else {
			//in this context, ignore limitation in db query. Limitation is handled in resolveEntityByQueryResult()
			//save limit as temp var and unset limit. This is important for translations.
			$limit = $this->limit;
			$this->unsetLimit();

			array_unshift($constructorParams, $this);
			array_unshift($constructorParams, $queryResultClass);
			$result = call_user_func_array(array($this->objectManager, 'get'), $constructorParams);

			$result = $this->resolveEntityByQueryResult($result, $limit);

			return $result;
		}
	}

	/**
	 *
	 * this function transforms the pages in form of the default model in queryResult to its proper doktype models
	 *
	 *
	 * @param \FORM4\Form4Pages\Persistence\QueryResult $queryResult
	 * @param \integer $limit
	 * @return \FORM4\Form4Pages\Persistence\QueryResult
	 */
	protected function resolveEntityByQueryResult($queryResult, $limit = 0) {
		$finalResultArray = array();

		foreach ($queryResult as $key => $object) {
			//get the page overlay will be required later
			$pageOverlay = $this->pageRepository->getPageOverlay($this->persistenceManager->getIdentifierByObject($object), $GLOBALS['TSFE']->sys_language_uid);

			if ($GLOBALS['TSFE']->sys_language_uid == 0) {
				//if current language is default: just proceed
				if ($this->resolveEntitiesToDoktype) {
					$finalResultArray[] = $this->persistenceManager->getObjectByIdentifier($this->persistenceManager->getIdentifierByObject($object), $this->getDoktypeEntity($object->getDoktype()));
				} else {
					$finalResultArray[] = $object;
				}
			} elseif ($GLOBALS['TSFE']->sys_language_uid > 0 && !empty($pageOverlay)) {
				//if we're in a translation context and pagerOverlay has a result: proceed
				if ($this->resolveEntitiesToDoktype) {
					$finalResultArray[] = $this->persistenceManager->getObjectByIdentifier($this->persistenceManager->getIdentifierByObject($object), $this->getDoktypeEntity($object->getDoktype()));
				} else {
					$finalResultArray[] = $object;
				}
			} elseif ($GLOBALS['TSFE']->sys_language_contentOL === 'hideNonTranslated' || $GLOBALS['TSFE']->sys_language_mode === 'strict') {
				//we're in a translation context but page Overlay was not found. If hidden is activated remove this page from results
			} else {
				if ($this->resolveEntitiesToDoktype) {
					//pages are not hidden: therefore proceed
					$finalResultArray[] = $this->persistenceManager->getObjectByIdentifier($this->persistenceManager->getIdentifierByObject($object), $this->getDoktypeEntity($object->getDoktype()));
				} else {
					$finalResultArray[] = $object;
				}
			}

			//check if limit is reached then stop iteration
			if ($limit) {
				if (count($finalResultArray) >= $limit) {
					break;
				}
			}
		}
		$queryResult->setQueryResult($finalResultArray);
		return $queryResult;
	}

}
?>