<?php
namespace FORM4\Form4Pages\Utility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2011-2013 Ingmar Schlecht <ingmar@typo3.org>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *  A copy is found in the textfile GPL.txt and important notices to the license
 *  from the author is found in LICENSE.txt distributed with these scripts.
 *
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 *
 *
 * @author	Johannes Härtner <johannes.haertner@form4.de>
 * @author	Michael Hübe <typo3@form4.de>
 * @author	Thomas Grothaus <thomas.grothaus@form4.de>
 * @author	Stephan Jorek <stephan.jorek@form4.de>
 * @package form4_pages
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class ConfigurationUtility implements \TYPO3\CMS\Core\SingletonInterface {

	/**
	 *
	 * @var string
	 */
	const _EXTKEY = 'form4_pages';

	/**
	 *
	 * @var array
	 */
	static public $DOKTYPES = array();

	/**
	 * Creates dynamic available doktypes items.
	 *
	 * @param array $config FlexForm configuration array.
	 * @return array FF FlexForm
	 */
	public function pageDoktypeItemProcFunc(array $config, $allowedDoktypes = NULL) {
		global $TCA, $LANG;

		if (!isset($config["items"])) {
			$config["items"] = array();
		}

		foreach($TCA["pages"]["columns"]["doktype"]["config"]["items"] as $key => $item) {
			$item[0] = $LANG->sL($item[0]);
			$config["items"][] = $item;
		}

		if ($allowedDoktypes) {
			$previousWasDivider = FALSE;
			$config['items'] = array_filter(array_reverse($config['items']), function($item) use($allowedDoktypes, &$previousWasDivider) {
				if ($item[1] == '--div--') {
					if ($previousWasDivider) {
						return FALSE;
					}
					return $previousWasDivider = TRUE;
				}
				if (GeneralUtility::inList($allowedDoktypes, (string) $item[1])) {
					$previousWasDivider = FALSE;
					return TRUE;
				}
				return FALSE;
			});
			reset($config['items']);
			while (!empty($config['items']) && $config['items'][key($config['items'])][1] == '--div--') {
				array_shift($config['items']);
			}
			$config['items'] = array_reverse($config['items']);
		}

		return $config;
	}
	
	/**
	 * Registers a new doktype.
	 * 
	 * @param integer $key Doktype key
	 * @param string $label Label used in backend
	 * @param string $icon Icon used in backend
	 * @param integer $copyTcaType TCA type configuration that is copied for the new doktype
	 * @param boolean $noOverlay Do not add the doktype to pages_language_overlay
	 * @return void
	 */
	public static function addDoktype($key, $label, $icon, $copyTcaType = 1, $noOverlay = FALSE) {
		global $TYPO3_CONF_VARS, $TCA;
		
		// add to FE configuration
		$TYPO3_CONF_VARS['FE']['content_doktypes'] .= ',' . $key;
		
		// register doktype icon
		$iconForSpritemanager = str_replace(
			PATH_site,
			'../',
			 GeneralUtility::getFileAbsFileName($icon)
		);
		\TYPO3\CMS\Backend\Sprite\SpriteManager::addTcaTypeIcon('pages', $key, $iconForSpritemanager);
		
		// copy existing TCA type configuration for new doktype
		$TCA['pages']['types'][$key]['showitem'] = $TCA['pages']['types'][$copyTcaType]['showitem'];
		// add doktype as item to field pages.doktype
		$TCA['pages']['columns']['doktype']['config']['items'][] = array($label, $key, $icon);
		
		// the same for overlay
		if (!$noOverlay) {
			// copy existing TCA type configuration for new doktype
			$TCA['pages_language_overlay']['types'][$key]['showitem'] = $TCA['pages_language_overlay']['types'][$copyTcaType]['showitem'];
			// add doktype as item to field pages.doktype
			$TCA['pages_language_overlay']['columns']['doktype']['config']['items'][] = array($label, $key, $icon);
		}
	}
	
	/**
	 * Adds a divider to the doktype selector.
	 * 
	 * @param string $label Label of the divider
	 * @param boolean $noOverlay Do not add the divider to pages_language_overlay
	 */
	public static function addDoktypeSelectorDivider($label, $noOverlay = FALSE) {
		global $TCA;
		$TCA['pages']['columns']['doktype']['config']['items'][] = array($label, '--div--');
		if (!$noOverlay) {
			$TCA['pages_language_overlay']['columns']['doktype']['config']['items'][] = array($label, '--div--');
		}
	}
	
	
	
	/**
	 *
	 * @param string $key
	 * @param string $name
	 * 
	 * @deprecated since 0.2.6. Will be removed in 0.4.0 - use ::addDoktype() instead
	 */
	public static function addPageDoktype($key, $name, $_EXTKEY = NULL) {
		
		GeneralUtility::logDeprecatedFunction();

		global $TCA, $TYPO3_CONF_VARS, $PAGES_TYPES, $CUSTOM_PAGE_TYPES;

		if ($_EXTKEY === NULL) {
			$_EXTKEY = self::_EXTKEY;
		}

		if (!is_array($CUSTOM_PAGE_TYPES)) {
			$CUSTOM_PAGE_TYPES = array();
		}

		$TYPO3_CONF_VARS['FE']['content_doktypes'] .= ',' . $key;

		$label = 'LLL:EXT:' . $_EXTKEY . '/Resources/Private/Language/locallang_db.xml:pages.doktype.' . $name;
		$icon  = \TYPO3\CMS\Core\Utility\ExtensionManagementUtility::extRelPath($_EXTKEY) . 'Resources/Public/Icons/' . $name . '.png';

		$CUSTOM_PAGE_TYPES[$key] = $PAGES_TYPES['default'];

		
		
		\TYPO3\CMS\Backend\Sprite\SpriteManager::addTcaTypeIcon('pages', $key, $icon);

		$TCA['pages']['types'][$key]['showitem'] = $TCA['pages']['types'][1]['showitem'];

		self::$DOKTYPES[] = array($label, $key, $icon);

	}

	/**
	 *
	 * @param string $_EXTKEY
	 * @return boolean
	 * 
	 * @deprecated since 0.2.6. Will be removed in 0.4.0 - use ::addDoktype() instead
	 */
	public static function applyConfiguration($_EXTKEY = NULL) {

		GeneralUtility::logDeprecatedFunction();

		if (empty(self::$DOKTYPES)) {
			return FALSE;
		}

		global $TCA, $TYPO3_CONF_VARS, $PAGES_TYPES, $CUSTOM_PAGE_TYPES;

		if ($_EXTKEY === NULL) {
			$_EXTKEY = self::_EXTKEY;
		}

		foreach (array('pages', 'pages_language_overlay') as $table) {

			foreach ($CUSTOM_PAGE_TYPES as $doktype => $conf) {
				if ($table == 'pages') {
					$CUSTOM_PAGE_TYPES[$doktype]['showitem'] = $TCA['pages']['types'][$doktype]['showitem'];
				}
			}


			$TCA[$table]['columns']['doktype']['config']['items'][] = array(
				'LLL:EXT:' . $_EXTKEY . '/Resources/Private/Language/locallang_db.xml:pages.' . $_EXTKEY . '.divider',
				'--div--'
			);

			foreach (self::$DOKTYPES as $doktype) {
				$TCA[$table]['columns']['doktype']['config']['items'][] = $doktype;
			}

			$TCA[$table]['types'] = GeneralUtility::array_merge($TCA[$table]['types'], $CUSTOM_PAGE_TYPES);
		}

		$PAGES_TYPES = GeneralUtility::array_merge($CUSTOM_PAGE_TYPES, (array) $PAGES_TYPES);

		self::$DOKTYPES = array();

		return TRUE;
	}
}
?>